#!/bin/sh
#####################
## Train the model ##
#####################

if [ $# -eq 0 ]
then
echo "Please specify the language. \n\nUsage: ./experiment.sh <en/zh>"
exit 1
fi

LANG=$1

if [ $1 != "en" ] && [ $1 != "zh" ]
then
echo "Please specify the language correctly. \n\nUsage: ./experiment.sh <en/zh>"
exit 1
fi

WORKING_DIR=$PWD
CONFIG=$WORKING_DIR/$LANG.cfg
EXP_DIR=$WORKING_DIR/experiments/$LANG/lambdagen/run_0/

mkdir -p $WORKING_DIR/experiments/$LANG/
mkdir -p $WORKING_DIR/experiments/$LANG/lambdagen/

rm -rf $EXP_DIR
mkdir $EXP_DIR

# this work:

echo "###################################"
echo "## Step 1 - Preparing the data   ##"
echo "###################################"

java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaReader $CONFIG

echo "###################################"
echo "## Step 2 - Training the model   ##"
echo "###################################"

java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaTrainer $CONFIG

echo "###################################"
echo "## Step 3 - Extracting the rules ##"
echo "###################################"

java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaExtractor $CONFIG

echo "###################################"
echo "## Step 4 - Learning the LM      ##"
echo "###################################"

java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaTrainLM $CONFIG

echo "###################################"
echo "## Step 5 - MERT                 ##"
echo "###################################"

echo "
top_n=100

LM 0.9999999999999999
RgivenE 1.5661521531337046
EgivenR 0.7997953541775705
WordP -2.3173966030273756

" > $EXP_DIR/lambdagen.cfg

echo "
LM		|||	1.0	Opt	0.1	+Inf	0.5	1.5
RgivenE	|||	4.73906927913536	Opt	-Inf	+Inf	-1	+1
EgivenR	|||	3.52225415320156	Opt	-Inf	+Inf	-1	+1
WordP	|||	-2.62415863048975	Opt	-Inf	+Inf	-1	+1
normalization = absval 1 LM
" > $EXP_DIR/params.txt

echo "
### Commonly used parameters
-r	$WORKING_DIR/data/geo-lambda/split/$LANG/tune.tgt                     # target sentences file name (in this case, file name prefix)
-rps	1                       # references per sentence
-p	$EXP_DIR/params.txt              # parameter file
-m	BLEU 4 closest          # evaluation metric and its options
-maxIt	15                      # maximum MERT iterations
-ipi	20                      # number of intermediate initial points per iteration
-cmd	$EXP_DIR/lambdagen.cmd          # file containing commands to run decoder
-decOut	$EXP_DIR/nbest.txt               # file prodcued by decoder
-dcfg	$EXP_DIR/lambdagen.cfg        # decoder config file
-N	100                      # size of N-best list generated each iteration
-v	2                       # verbosity level (0-2; higher value => more verbose)
-seed   12341234                # random number generator seed
" > $EXP_DIR/zmert.cfg

echo "java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaGen $CONFIG tune $EXP_DIR/lambdagen.cfg $EXP_DIR/nbest.txt" > $EXP_DIR/lambdagen.cmd

chmod +x $EXP_DIR/lambdagen.cmd

rm $EXP_DIR/nbest*
rm $EXP_DIR/*ZMERT*

java -Xmx2g -cp lambdagen.jar zmert.ZMERT $EXP_DIR/zmert.cfg

echo "###################################"
echo "## Step 6 - Generating           ##"
echo "###################################"

java -Xmx2g -cp lambdagen.jar lambdagen.main.LambdaGen $CONFIG test $EXP_DIR/lambdagen.cfg.ZMERT.final $EXP_DIR/nbest-test.txt

java -Xmx2g -cp lambdagen.jar -Dfile.encoding=UTF8 lambdagen.util.ExtractTopCand $EXP_DIR/nbest-test.txt $EXP_DIR/1best-test.txt

echo "The output is stored here: $EXP_DIR/1best-test.txt"

echo "###################################"
echo "## Step 7 - Evaluating           ##"
echo "###################################"

java -cp lambdagen.jar lambdagen.util.Eval \
	-cand $EXP_DIR/1best-test.txt \
	-ref $WORKING_DIR/data/geo-lambda/split/$LANG/test.tgt \
	-rps 1 -m BLEU 4 closest -txtNrm 0 > $EXP_DIR/bleu.txt

grep "BLEU =" $EXP_DIR/bleu.txt
