/*
 * Trace.java
 *
 * Created on June 5, 2007, 5:42 PM
 *
 */

package lambdacalc.lf;

import lambdacalc.logic.Expr;
import lambdacalc.logic.GApp;

/**
 * A trace or a pronoun.
 *
 * @author champoll
 */
public class Trace extends Terminal {
    
    public static final String SYMBOL = "t";
    
    public Trace(String label, int index) {
        super(label, index);
    }
    
    public Trace(int index) {
        this(SYMBOL, index);
    }
    
    @Override
	public boolean isMeaningful() {
        return true;
    }
    
    public boolean isActualTrace() {
        return this.getLabel().equals(Trace.SYMBOL);
    }
    
    @Override
	public Expr getMeaning(AssignmentFunction g) throws MeaningEvaluationException {
        if (g == null)
            return new GApp(this.getIndex());
        else
            return (Expr)g.get(getIndex());
    }    

    @Override
	public void setLabel(String label) {
        throw new UnsupportedOperationException("Tried to set the label of a trace.");
    }
    
    @Override
	public void setIndex(int index) {
        if (index == -1) {
            throw new UnsupportedOperationException("Tried to remove the index of a trace.");
        }
        super.setIndex(index);
    }
    
    @Override
	public void removeIndex() {
        throw new UnsupportedOperationException("Tried to remove the index of a trace.");
    }
    
    @Override
	public String getDisplayName() {
        return "Trace";
    }

    /**
     * Nothing to do on a Trace.
     *
     * @param lexicon the lexicon
     */
    @Override
	public void guessLexicalEntries(Lexicon lexicon) {
    
    }
    
}
