package lambdacalc.logic;
/*
 * ArgMin.java
 *
 * Created on May 29, 2006, 3:34 PM
 */


import java.awt.event.KeyEvent;

/**
 * Represents the lambda binder.
 */
public class ArgMin extends Binder {
    /**
     * The unicode Greek letter lambda.
     */
    public static final char SYMBOL = '\u03C3';

    public static final char INPUT_SYMBOL = 'U';

    public static final int KEY_EVENT = KeyEvent.VK_L;

    /**
     * Constructs the binder.
     * @param ident the identifier the binder binds, which may
     * be a constant to capture student errors.
     * @param innerExpr the inner expression
     * @param hasPeriod indicates whether this binder's string
     * representation includes a period after the identifier.
     */
    public ArgMin(Identifier ident, Expr innerExpr, boolean hasPeriod) {
        super(ident,innerExpr,hasPeriod);
    }

    @Override
	public int getExprCat(){
        return super.ARGMIN;
    }

    /**
     * Gets the operator precedence of this operator.
     * All values are documented in Expr, so don't change the value here
     * without changing it there.
     */
    @Override
	public int getOperatorPrecedence() {
        return 4;
    }

    @Override
	public boolean dotPolicy() {
        return true;
    }

    @Override
	public String getSymbol() {
        return String.valueOf(SYMBOL);
    }

    @Override
	public Type getType() throws TypeEvaluationException {
        checkVariable();
        if(!(this.getInnerExpr().getClass().getName().equals("lambdacalc.logic.FunApp"))){
            throw new RuntimeException(this.getInnerExpr().getClass().getName());
        } else {
            Expr func = ((FunApp)this.getInnerExpr()).getFunc();
            Expr args = ((FunApp)this.getInnerExpr()).getArg();
            Expr[] theArgs = ((ArgList)args).getArgs();
            if(theArgs.length!=2){
                throw new TypeEvaluationException("There should be exactly two args for ArgMin!");
            }
            if(!theArgs[0].getType().equals(Type.ET)){
                throw new TypeEvaluationException("The first argument for ArgMin should have type t. Found:"+theArgs[0].getType());
            }
            if(!theArgs[1].getType().equals(Type.EE)){
                throw new TypeEvaluationException("The second argument for ArgMin should have type e. Found:"+theArgs[1].getType());
            }
        }

        //return new CompositeType(getVariable().getType(), getInnerExpr().getType());
        return getVariable().getType();
    }

    @Override
	protected Binder create(Identifier variable, Expr inner) {
        return new ArgMin(variable, inner, hasPeriod());
    }

    ArgMin(java.io.DataInputStream input) throws java.io.IOException {
        super(input);
    }
}
