/*
 * Equality.java
 *
 * Created on June 1, 2006, 2:48 PM
 */

package lambdacalc.logic;

/**
 * Equality and inequality operators "=" and "=/=".
 */
public class Equality extends LogicalBinary {
    /**
     * The equal sign character
     */
    public static final char EQ_SYMBOL = '=';

    /**
     * The not-equal character
     */
    public static final char NEQ_SYMBOL = '\u2260';

    public static final char LT_SYMBOL = '<';

    public static final char GT_SYMBOL = '>';

    private int equality;

    /**
     * Constructs the connective.
     * @param left the expression on the left side of the connective
     * @param right the expression on the right side of the connective
     * @param equality true for an equality operator, false for inequality
     */
    public Equality(Expr left, Expr right, int equality) {
        super(left, right);
        this.equality = equality;
    }

    @Override
	public int getExprCat() {
        return super.EQUALITY;
    }

    public int getEquality() {
        return this.equality;
    }

    @Override
	public String getSymbol() {
        if (equality == 0) {
            return String.valueOf(EQ_SYMBOL);
        } else if (equality == 1) {
            return String.valueOf(NEQ_SYMBOL);
        } else if (equality == 2) {
            return String.valueOf(LT_SYMBOL);
        } else {
            return String.valueOf(GT_SYMBOL);
        }
    }

    @Override
	public Type getType() throws TypeEvaluationException {
        if (!getLeft().getType().equals(getRight().getType())) {
            throw new TypeMismatchException("The types of the expressions on the left and right of an equality operator must be the same, but " +
                                            getLeft() + " is of type " + getLeft().getType() + " and " + getRight() + " is of type " +
                                            getRight().getType() + ".");
        }
        return Type.T;
    }

    @Override
	protected boolean equalsHelper(Binary b) {
        return b instanceof Equality && equality == ((Equality) b).equality;
    }

    @Override
	protected Binary create(Expr left, Expr right) {
        return new Equality(left, right, equality);
    }

    @Override
	public void writeToStream(java.io.DataOutputStream output) throws java.io.IOException {
        super.writeToStream(output);
        output.writeShort(0); // data format version
        output.writeInt(equality);
    }

    Equality(java.io.DataInputStream input) throws java.io.IOException {
        super(input);
        if (input.readShort() != 0) {
            throw new java.io.IOException("Invalid data."); // future version?
        }
        equality = input.readInt();
    }
}
