/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.data;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import lambdacalc.logic.Expr;

/**
 * @author Wei LU
 * @version 1.0
 */

public class ExprToken extends Token{
	
	private static final long serialVersionUID = 7490812234205131290L;
	
	private Expr _exp;
	
	public ExprToken(Expr exp){
		if(exp==null){
			throw new RuntimeException("The expression is null! Error code: 029");
		}
		this._exp = exp;
	}
	
	public Expr getExpr(){
		return this._exp;
	}
	
	@Override
	public int hashCode(){
		return this._exp.hashCode() + 12345;
	}
	
	//equals if the two expressions are alpha equivalent...
	@Override
	public boolean equals(Object o){
		if(o instanceof ExprToken){
			ExprToken token = (ExprToken)o;
			return this._exp.alphaEquivalent(token._exp);
		}
		return false;
	}
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		sb.append(this._exp.toString());
		return sb.toString();
	}
	
	@Override
	public int compareTo(Token t) {
		if(t instanceof ExprToken){
			ExprToken et = (ExprToken)t;
			if(this._id == -1 || et._id == -1){
				throw new RuntimeException("Error: comparison of ExprToken not ready!");
			}
			return this._id - et._id;
		}
		throw new RuntimeException("Invalid comparison between an ExprToken and a "+t.getClass().getName());
	}
	
	/**
	 * Private methods
	 */
	
	/**
	 * Serialization.
	 */
	
	private void writeObject(ObjectOutputStream out)throws IOException{
		out.writeInt(this._id);
		out.writeObject(this._exp);
		out.flush();
	}
	
	private void readObject(ObjectInputStream in)throws IOException, ClassNotFoundException{
		this._id = in.readInt();
		this._exp = (Expr)in.readObject();
	}
	
}
