/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.data;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Hashtable;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaPattern implements Serializable{

	private static final long serialVersionUID = -3742029065734174932L;
	
	private int _id;
    private String _p;
    private int _len;
    private int _nid;//id used for accessing from the complete list


    /**
     * 1 - X
     * 2 - Y
     * 9 - w
     */
    public static final LambdaPattern _w = new LambdaPattern(9, "w");

    public static final LambdaPattern _X = new LambdaPattern(1, "X");
    public static final LambdaPattern _wX = new LambdaPattern(91, "wX");
    public static final LambdaPattern _Xw = new LambdaPattern(19, "Xw");
    public static final LambdaPattern _wXw = new LambdaPattern(919, "wXw");
    
    public static final LambdaPattern _Y = new LambdaPattern(2, "Y");
    public static final LambdaPattern _wY = new LambdaPattern(92, "wY");
    public static final LambdaPattern _Yw = new LambdaPattern(29, "Yw");
    public static final LambdaPattern _wYw = new LambdaPattern(929, "wYw");
    
    public static final LambdaPattern _XY = new LambdaPattern(12, "XY");
    
    public static final LambdaPattern _wXY = new LambdaPattern(912, "wXY");
    public static final LambdaPattern _XwY = new LambdaPattern(192, "XwY");
    public static final LambdaPattern _XYw = new LambdaPattern(129, "XYw");
    
    public static final LambdaPattern _wXwY = new LambdaPattern(9192, "wXwY");
    public static final LambdaPattern _wXYw = new LambdaPattern(9129, "wXYw");
    public static final LambdaPattern _XwYw = new LambdaPattern(1929, "XwYw");
    
    public static final LambdaPattern _wXwYw = new LambdaPattern(91929, "wXwYw");
    
    public static final LambdaPattern _YX = new LambdaPattern(21, "YX");
    
    public static final LambdaPattern _wYX = new LambdaPattern(921, "wYX");
    public static final LambdaPattern _YwX = new LambdaPattern(291, "YwX");
    public static final LambdaPattern _YXw = new LambdaPattern(219, "YXw");
    
    public static final LambdaPattern _wYwX = new LambdaPattern(9291, "wYwX");
    public static final LambdaPattern _wYXw = new LambdaPattern(9219, "wYXw");
    public static final LambdaPattern _YwXw = new LambdaPattern(2919, "YwXw");
    
    public static final LambdaPattern _wYwXw = new LambdaPattern(92919, "wYwXw");
    
    public boolean isReversed(){
    	int Xindex = this._p.indexOf("X");
    	int Yindex = this._p.indexOf("Y");
    	if(Xindex >=0 && Yindex>=0 && Xindex > Yindex){
    		return true;
    	}
    	return false;
    }
    
    public int getArity(){
    	if(this.equals(_w)){
    		return 0;
    	} else if(this._p.indexOf("X") >= 0 && this._p.indexOf("Y") >= 0){
    		return 2;
    	} else {
    		return 1;
    	}
    }
    
    public static final LambdaPattern[] _patterns = new LambdaPattern[]{_w, 
    	_X, _wX, _Xw, _wXw, 
    	_Y, _wY, _Yw, _wYw, 
    	_XY, _wXY, _XwY, _XYw, _wXwY, _wXYw, _XwYw, _wXwYw, 
    	_YX, _wYX, _YwX, _YXw, _wYwX, _wYXw, _YwXw, _wYwXw};
    static {
    	for(int i =0; i< _patterns.length; i++){
    		_patterns[i]._nid = i;
    	}
    }
    
    public static final LambdaPattern[] _patterns_arity_2 = new LambdaPattern[]{
    	_XY, _wXY, _XwY, _XYw, _wXwY, _wXYw, _XwYw, _wXwYw, 
    	_YX, _wYX, _YwX, _YXw, _wYwX, _wYXw, _YwXw, _wYwXw
    };
    
    public static final LambdaPattern[] _patterns_arity_1 = new LambdaPattern[]{
    	_X, _wX, _Xw, _wXw
    };
    
    private static final Hashtable<Integer, LambdaPattern> id2pattern = new Hashtable<Integer, LambdaPattern>();
    static{
    	for(LambdaPattern LambdaPattern: _patterns){
    		id2pattern.put(LambdaPattern.getId(), LambdaPattern);
    	}
    }
    
    private static final Hashtable<Integer, LambdaPattern[]> splitTable_2 = new Hashtable<Integer, LambdaPattern[]>();
    static{
    	splitTable_2.put(_wX._id, new LambdaPattern[] {_w, _X});
    	splitTable_2.put(_wY._id, new LambdaPattern[] {_w, _Y});
    	splitTable_2.put(_Xw._id, new LambdaPattern[] {_X, _w});
    	splitTable_2.put(_Yw._id, new LambdaPattern[] {_Y, _w});
    	
    	splitTable_2.put(_wXw._id, new LambdaPattern[] {_w, _Xw});
    	splitTable_2.put(_wYw._id, new LambdaPattern[] {_w, _Yw});
        
    	splitTable_2.put(_XY._id, new LambdaPattern[] {_X, _Y});
    	splitTable_2.put(_YX._id, new LambdaPattern[] {_Y, _X});
        
    	splitTable_2.put(_wXY._id, new LambdaPattern[] {_wX, _Y});
    	splitTable_2.put(_wYX._id, new LambdaPattern[] {_wY, _X});
        splitTable_2.put(_XwY._id, new LambdaPattern[] {_Xw, _Y});
    	splitTable_2.put(_YwX._id, new LambdaPattern[] {_Yw, _X});
        splitTable_2.put(_XYw._id, new LambdaPattern[] {_X, _Yw});
    	splitTable_2.put(_YXw._id, new LambdaPattern[] {_Y, _Xw});
    	
        splitTable_2.put(_wXwY._id, new LambdaPattern[] {_wX, _wY});
        splitTable_2.put(_wYwX._id, new LambdaPattern[] {_wY, _wX});
        splitTable_2.put(_wXYw._id, new LambdaPattern[] {_wX, _Yw});
    	splitTable_2.put(_wYXw._id, new LambdaPattern[] {_wY, _Xw});
        splitTable_2.put(_XwYw._id, new LambdaPattern[] {_Xw, _Yw});
    	splitTable_2.put(_YwXw._id, new LambdaPattern[] {_Yw, _Xw});
        splitTable_2.put(_wXwYw._id, new LambdaPattern[] {_wXw, _Yw});
    	splitTable_2.put(_wYwXw._id, new LambdaPattern[] {_wYw, _Xw});
    }
    
    private static final Hashtable<Integer, LambdaPattern[][]> splitTable = new Hashtable<Integer, LambdaPattern[][]>();
    static{
    	splitTable.put(_wX._id, new LambdaPattern[][] {{_w, _X},{_w, _wX}});
    	splitTable.put(_wY._id, new LambdaPattern[][] {{_w, _Y},{_w, _wY}});

    	splitTable.put(_Xw._id, new LambdaPattern[][] {{_X, _w},{_Xw, _w}});
    	splitTable.put(_Yw._id, new LambdaPattern[][] {{_Y, _w},{_Yw, _w}});
    	
    	splitTable.put(_wXw._id, new LambdaPattern[][] {{_w, _Xw},{_w, _wXw}});
    	splitTable.put(_wYw._id, new LambdaPattern[][] {{_w, _Yw},{_w, _wYw}});
    }
    
    private static final Hashtable<Integer, LambdaPattern[]> superRelevanceTable = new Hashtable<Integer, LambdaPattern[]>();
    static{
    	superRelevanceTable.put(_wX._id, new LambdaPattern[] {_wX, _wXw});
    	superRelevanceTable.put(_wXw._id, new LambdaPattern[] {_wXw});
    	superRelevanceTable.put(_Xw._id, new LambdaPattern[] {_Xw, _wXw});
    	//TODO... this is not finished..
    }
    
    public static LambdaPattern[] split2(LambdaPattern p){

    	splitTable_2.put(_wX._id, new LambdaPattern[] {_w, _X});
    	splitTable_2.put(_wY._id, new LambdaPattern[] {_w, _Y});
    	splitTable_2.put(_Xw._id, new LambdaPattern[] {_X, _w});
    	splitTable_2.put(_Yw._id, new LambdaPattern[] {_Y, _w});
    	
    	splitTable_2.put(_wXw._id, new LambdaPattern[] {_w, _Xw});
    	splitTable_2.put(_wYw._id, new LambdaPattern[] {_w, _Yw});
        
    	splitTable_2.put(_XY._id, new LambdaPattern[] {_X, _Y});
    	splitTable_2.put(_YX._id, new LambdaPattern[] {_Y, _X});
        
    	splitTable_2.put(_wXY._id, new LambdaPattern[] {_wX, _Y});
    	splitTable_2.put(_wYX._id, new LambdaPattern[] {_wY, _X});
        splitTable_2.put(_XwY._id, new LambdaPattern[] {_Xw, _Y});
    	splitTable_2.put(_YwX._id, new LambdaPattern[] {_Yw, _X});
        splitTable_2.put(_XYw._id, new LambdaPattern[] {_X, _Yw});
    	splitTable_2.put(_YXw._id, new LambdaPattern[] {_Y, _Xw});
    	
        splitTable_2.put(_wXwY._id, new LambdaPattern[] {_wX, _wY});
        splitTable_2.put(_wYwX._id, new LambdaPattern[] {_wY, _wX});
        splitTable_2.put(_wXYw._id, new LambdaPattern[] {_wX, _Yw});
    	splitTable_2.put(_wYXw._id, new LambdaPattern[] {_wY, _Xw});
        splitTable_2.put(_XwYw._id, new LambdaPattern[] {_Xw, _Yw});
    	splitTable_2.put(_YwXw._id, new LambdaPattern[] {_Yw, _Xw});
        splitTable_2.put(_wXwYw._id, new LambdaPattern[] {_wXw, _Yw});
    	splitTable_2.put(_wYwXw._id, new LambdaPattern[] {_wYw, _Xw});
    	LambdaPattern[] results = splitTable_2.get(p.getId());
    	if(results == null){
    		throw new RuntimeException("splitting 2 of LambdaPattern " + p + " not supported!");
    	} else {
    		return results;
    	}
    }
    
    public static LambdaPattern[][] split(LambdaPattern p){
    	LambdaPattern[][] results = splitTable.get(p.getId());
    	if(results == null){
    		throw new RuntimeException("splitting 1 of LambdaPattern " + p + " not supported!");
    	} else {
    		return results;
    	}
    }
    
    /**
     * Given a LambdaPattern p, you want to check what makes you want to care p.
     * @param p
     * @return
     */
    public LambdaPattern[] getSuperRelevance(LambdaPattern p){
    	//TODO incomplete
    	return null;
    }
    
    public static LambdaPattern getPatternById(int id){
    	LambdaPattern p = id2pattern.get(id);
    	return p;
    }

    public LambdaPattern(int id, String p) {
        this._id = id;
        this._p = p;
        this._len = p.length();
    }

    public int getId() {
        return this._id;
    }
    
    public int getNId(){
    	return this._nid;
    }

    public String getP() {
        return this._p;
    }

    public int getLen() {
        return this._len;
    }

    @Override
	public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("LambdaPattern:");
        sb.append(this._id);
        sb.append(":");
        sb.append(this._p);
        return sb.toString();
    }
    
    
    private void readObject(ObjectInputStream in)throws IOException, ClassNotFoundException{
        this._id = in.readInt();
        this._p = in.readUTF();
        this._len = in.readInt();
    }

    private void writeObject(ObjectOutputStream out) throws IOException, ClassNotFoundException{
        out.writeInt(this._id);
        out.writeUTF(this._p);
        out.writeInt(this._len);
    }
    
}
