/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.data;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import lambdacalc.logic.Type;

/**
 * @author Wei LU
 * @version 1.0
 */

public class TypeToken extends Token{
	
	private static final long serialVersionUID = 8847850917607346973L;
	
	private Type _type;
	
	public static TypeToken _rootToken = new TypeToken(null);
	static{_rootToken.setId(0);}
	
	public TypeToken(Type type){
		this._type = type;
	}
	
	public Type getType(){
		return this._type;
	}
	
	@Override
	public int hashCode(){
		return this._type.hashCode() + 23456;
	}
	
	@Override
	public boolean equals(Object o){
		if(o instanceof TypeToken){
			TypeToken node = (TypeToken)o;
			if(this._type == null || node._type == null){
				if(this._type == null && node._type == null){
					return true;
				}
				return false;
			}
			return this._type.equals(node._type);
		}
		return false;
	}
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		if(this._type==null){
			sb.append("[R]");
		} else {
			sb.append(this._type.toString().toUpperCase());
		}
		return sb.toString();
	}
	
	@Override
	public int compareTo(Token t) {
		if(t instanceof TypeToken){
			TypeToken tt = (TypeToken)t;
			if(this._id == -1 || tt._id == -1){
				throw new RuntimeException("Error: comparison of TypeToken not ready!");
			}
			return this._id - tt._id;
		}
		throw new RuntimeException("Invalid comparison between a TypeToken and a "+t.getClass().getName());
	}
	
	/**
	 * Private methods.
	 * For serialization
	 */
	private void writeObject(ObjectOutputStream out)throws IOException{
		out.writeInt(this._id);
		out.writeObject(this._type);
	}
	
	private void readObject(ObjectInputStream in)throws IOException, ClassNotFoundException{
		this._id = in.readInt();
		this._type = (Type)in.readObject();
	}
}
