/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.data;

import java.io.*;

/**
 * @author Wei LU
 * @version 1.0
 */

public class Word implements Comparable<Word>, Serializable {
	
    private static final long serialVersionUID = 3755185923068870075L;
    
    public static Word _X1;
    public static Word _X2;
    public static Word _UNK;
    public static Word _BEGIN;
    public static Word _END;
    
    private int _id = -1;
    private String _w;
    
    public Word(String w) {
        this._w = w;
    }
    
    @Override
	public int hashCode(){
    	return this._w.hashCode() * 12345;
    }
    
    @Override
	public boolean equals(Object o){
    	if(o instanceof Word){
    		Word word = (Word)o;
        	return this._w.equals(word._w);
    	}
    	return false;
    }
    
    @Override
	public int compareTo(Word word) {
    	return this._w.compareTo(word._w);
    }
    
    public void setId(int id){
    	this._id = id;
    }
    
    public int getId() {
        return this._id;
    }
    
    public String getWord() {
        return this._w;
    }
    
    @Override
	public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append(this._w);
        sb.append('{');
        sb.append(this._id);
        sb.append('}');
        return sb.toString();
    }
    
    /**
     * Private methods...
     */
    /**
     * Serialization methods...
     */
    private void readObject(ObjectInputStream in) throws IOException {
        this._id = in.readInt();
        this._w = in.readUTF();
    }
    
    private void writeObject(ObjectOutputStream out) throws IOException, ClassNotFoundException {
        out.writeInt(this._id);
        out.writeUTF(this._w);
    }

}
