/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.gen;

import java.util.ArrayList;
import java.util.Arrays;

import lambdagen.data.LambdaProd;
import lambdagen.grammar.LambdaGRule;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaGenCell {
	
	public LambdaGRule _grule;
	private ArrayList<LambdaProd> _prods;
	private ArrayList<LambdaGenSpan[]> _spans;//each prod is associated with a span array, with the same index.
	private int _K;
	
	public LambdaGenCell(LambdaGRule grule, int K){
		this._grule = grule;
		this._prods = new ArrayList<LambdaProd>();
		this._spans = new ArrayList<LambdaGenSpan[]>();
		this._K = K;
	}
	
	public boolean addSpan(LambdaGenSpan span, LambdaProd prod){
		
		LambdaGenSpan[] spans = this.getSpansByProd(prod);
		
		if(span.getWeightedTotalScore() == Double.NEGATIVE_INFINITY){
			return false;
		}
		int index = Arrays.binarySearch(spans, span);
        if (index < 0) {
            index = -1 - index;
        }
        if (index == 0) {
            return false;
        }
        index = index - 1;
        //insertion
        for (int k = 0; k <= index - 1; k++) {
        	spans[k] = spans[k + 1];
        }
        spans[index] = span;
        
        return true;
        
	}
	
	public LambdaGenSpan[] getSpansByProd(LambdaProd prod){
		
		int index = this._prods.indexOf(prod);
		if(index < 0){
			//if the spans are not yet there.
			LambdaGenSpan[] spans = new LambdaGenSpan[this._K];
			Arrays.fill(spans, LambdaGenSpan._empty);
			this._prods.add(prod);
			this._spans.add(spans);
			return spans;
		} else {
			return this._spans.get(index);
		}
		
	}
	
	public LambdaGRule getGRule(){
		return this._grule;
	}
	
	public ArrayList<LambdaProd> getProds(){
		return this._prods;
	}
	
	public ArrayList<LambdaGenSpan[]> getSpans(){
		return this._spans;
	}

}
