/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.gen;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import lambdagen.data.Instance;
import lambdagen.data.Word;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaLM {

	private String _lm_file;
	private short _n; //ngram
	
	private JavaLM _lm;
	
	public LambdaLM(String lm_file, short n){
		this._lm_file = lm_file;
		this._n = n;
		File f = new File(this._lm_file);
		if(f.exists()){
			this._lm = new JavaLM(this._lm_file);
		} else if(lm_file.equals("")){
			System.err.println("Invalid lm_file: <empty>");
			System.exit(1);
		}
	}
	
	/**
	 * Compute the normal n-gram probability.
	 * @param words
	 * @param n
	 * @return
	 * @throws IOException 
	 */
	public double computeNgram(String sent){
		return this._lm.getFinalLogProb(sent);
		//TODO ...
	}
	
	/**
	 * Excluding the first n-1 words.
	 */
	public double computeNgram_p(ArrayList<Word> words){
		//TODO ...
		return 1.0;
	}
	
	//command : ngram-count -order 3 -unk -kndiscount1 -kndiscount2 -kndiscount3 -text $train -lm $lmfile
	public void trainSrilm(String path_to_srilm, ArrayList<Instance> trainInstances, String modeldir) throws IOException {
		
		File train_txt = new File(modeldir+"/train.txt");
		PrintWriter p = new PrintWriter(train_txt, "UTF-8");
		
		for(Instance trainInstance: trainInstances){
			ArrayList<Word> words = trainInstance.getWords();
			for(int i = 0; i<words.size(); i++){
				if(i != 0){ p.print(' ');}
				p.print(words.get(i).getWord());
			}
			p.println();
		}
		p.close();
		p.flush();
		p.close();
		
		String cmd_str = path_to_srilm+"/ngram-count -order "+this._n+" -unk -kndiscount1 -kndiscount2 -kndiscount3 -kndiscount4 -kndiscount5 -text "+train_txt+" -lm "+this._lm_file;
		
		try {
			Process proc = Runtime.getRuntime().exec(cmd_str);
			int exitVal = proc.waitFor();
			if (exitVal != 0) {
				System.err.println(cmd_str+" terminates abnormally");
				throw new RuntimeException();
			}
			System.err.println("language model learned and saved to "+this._lm_file);
		} catch (InterruptedException e) {
			System.err.println(e);
		}
		
	}
	
}
