/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.gen;

import java.io.Serializable;
import java.util.ArrayList;

import lambdagen.data.Word;

/**
 * @author Wei LU
 * @version 1.0
 */

public class Sentence implements Serializable, Comparable<Sentence>{

	private static final long serialVersionUID = 1551400151289838544L;
	
	private ArrayList<Word> _words;
	private double _score;

	public Sentence(ArrayList<Word> words, double score){
		this._words = words;
		this._score = score;
	}
	
	public ArrayList<Word> getWords(){
		return this._words;
	}
	
	public double getScore(){
		return this._score;
	}
	
	/**
	 * To the string representation of the sentence, i.e. the English/Chinese natural language sentence.
	 */
	public String toSent(){
		StringBuffer sb = new StringBuffer();
		for(int i = 0 ; i< this._words.size(); i++){
			if(i != 0){
				sb.append(' ');
			}
			sb.append(this._words.get(i).getWord());
		}
		
		return sb.toString();
	}

	@Override
	public int compareTo(Sentence sent) {
		if(this._score > sent._score){
			return +1;
		} else if(this._score < sent._score){
			return -1;
		}
		return 0;
	}
	
}
