/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.gen;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import lambdagen.grammar.LambdaGRule;

/**
 * @author Wei LU
 * @version 1.0
 */

public class Weights implements Serializable{

	/**
	 * For normalization mode.
	 */
	public static final int _NORM_NONE = 5000; // no normalization...
	public static final int _NORM_LM = 10000; //default normalization method.
	public static final int _NORM_TRANS = 20000;
	public static final int _NORM_SUM = 30000; //not recommended...
	
	private static final long serialVersionUID = -4534482828600092676L;
	private double _lmWeight;
	private double[] _featureWeights;
	
	private int _mode = _NORM_LM;
	
	public Weights(double[] featureWeights, double lmWeight){
		this._featureWeights = featureWeights;
		this._lmWeight = lmWeight;
	}
	
	/**
	 * 
	 * @param parent
	 * @param children
	 * @param n : the n in "n-gram"
	 * @return
	 */
	public double computeLMScore(LambdaGRule parent, LambdaGRule[] children, int n){
		
		
		
		//TODO 
		return 0.0;
	}
	
	/**
	 * Compute the score associated with the gr alone.
	 */
	public double computeWeightedFeatureScore(LambdaGRule gr){
		double score = 0;
		double[] features = gr.getFeatures();
		for(int i =0; i<features.length; i++){
			score += features[i]*this._featureWeights[i];
		}
		return score;
	}
	
	public double computeWeightedFeatureScore(LambdaGenSpan span){
		double score = 0;
		double[] features = span.getFeatureScores();
		for(int i =0; i<features.length; i++){
			score += features[i]*this._featureWeights[i];
		}
		return score;
	}
	
	/**
	 * Set the new weights and then perform a normalization.
	 */
	public void setWeightsAndNorm(double featureWeights[], double lmWeight){
		this._featureWeights = featureWeights;
		this._lmWeight = lmWeight;
	}
	
	public void setFeatureWeights(double[] featureWeights){
		this._featureWeights = featureWeights;
	}
	
	public void setLMWeight(double lmWeight){
		this._lmWeight = lmWeight;
	}
	
	public double[] getFeatureWeights(){
		return this._featureWeights;
	}
	
	public double getLMWeight(){
		return this._lmWeight;
	}
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		
		sb.append("[lm weight]\t"+this._lmWeight);
		for(int i =0; i<this._featureWeights.length; i++){
			sb.append("[phrasetable"+i+" weight]\t"+this._featureWeights[i]);
		}
		
		return sb.toString();
	}

	private void writeObject(ObjectOutputStream out)throws IOException{
		out.writeObject(this._lmWeight);
		out.writeInt(this._featureWeights.length);
		for(int i =0; i<this._featureWeights.length; i++){
			out.writeDouble(this._featureWeights[i]);
		}
	}
	
	private void readObject(ObjectInputStream in)throws IOException{
		this._lmWeight = in.readDouble();
		this._featureWeights = new double[in.readInt()];
		for(int i =0; i<this._featureWeights.length; i++){
			this._featureWeights[i] = in.readDouble();
		}
	}
}
