/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.grammar;

import java.util.ArrayList;

import lambdagen.gen.LambdaGenCell;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaRule {
	
	private int _seq = -1;
	
	private LambdaGRule _gr;//the generic rule
	private ArrayList<LambdaRule> _children1 = null;
	private ArrayList<LambdaRule> _children2 = null;
	
	private ArrayList<LambdaRule> _allRules = null;//all rules with the root at this rule.
	
	private LambdaGenCell _cell;
	
	public LambdaGenCell getGenCell(){
		return this._cell;
	}
	
	public void setGenCell(LambdaGenCell cell){
		this._cell = cell;
	}
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		this.toStringHelper(0, sb);
		return sb.toString();
	}
	
	private void toStringHelper(int indent, StringBuffer sb){
		for(int i =0; i<indent; i++){
			sb.append('\t');
		}
		sb.append(this._gr.toString()+"\t("+this.countTrees()+")["+this._seq+"]");
		sb.append('\n');
		if(this._children1!=null){
			for(int i = 0 ; i<this._children1.size(); i++){
				sb.append("C1:"+i+":");
				this._children1.get(i).toStringHelper(indent+1, sb);
			}
		}
		if(this._children2!=null){
			for(int i = 0 ; i<this._children2.size(); i++){
				sb.append("C2:"+i+":");
				this._children2.get(i).toStringHelper(indent+1, sb);
			}
		}
	}

	public int countTrees(){
		int arity = this.getArity();
		if(arity == 0){
			return 1;
		} else if(arity == 1){
			int size = 0;
			for(int i = 0; i<this._children1.size(); i++){
				size += this._children1.get(i).countTrees();
			}
			return size;
		} else if(arity == 2){
			int size1 = 0;
			for(int i = 0; i<this._children1.size(); i++){
				size1 += this._children1.get(i).countTrees();
			}
			int size2 = 0;
			for(int i = 0; i<this._children2.size(); i++){
				size2 += this._children2.get(i).countTrees();
			}
			return size1*size2;
		}
		throw new RuntimeException("Invalid !! 44.");
	}
	
	public LambdaRule(LambdaGRule gr){
		this._gr = gr;
	}
	
	public int getArity(){
		if(this._children2 != null){
			return 2;
		} else if(this._children1 != null){
			return 1;
		} else {
			return 0;
		}
	}
	
	public void setSeq(int seq){
		this._seq = seq;
	}
	
	public int getSeq(){
		return this._seq;
	}
	
	public LambdaGRule getGRule(){
		return this._gr;
	}
	
	public ArrayList<LambdaRule> getFirstChildren(){
		return this._children1;
	}
	
	public ArrayList<LambdaRule> getSecondChildren(){
		return this._children2;
	}
	
	public void setChildren(ArrayList<LambdaRule> children){
		this._children1 = children;
	}
	
	public void setChildren(ArrayList<LambdaRule> children1, ArrayList<LambdaRule> children2){
		this._children1 = children1;
		this._children2 = children2;
	}
	
	public ArrayList<LambdaRule> getAllRules(){
		
		if(this._allRules== null){
			this._allRules = new ArrayList<LambdaRule>();

			int arity = this.getArity();
			if(arity >=2){
				for(int i = 0; i<this._children2.size(); i++){
					ArrayList<LambdaRule> cRules = this._children2.get(i).getAllRules();
					for(LambdaRule cRule : cRules){
						this._allRules.add(cRule);
					}
				}
			}
			if(arity >=1){
				for(int i = 0; i<this._children1.size(); i++){
					ArrayList<LambdaRule> cRules = this._children1.get(i).getAllRules();
					for(LambdaRule cRule : cRules){
						this._allRules.add(cRule);
					}
				}
			}
			this._allRules.add(this);
		}
		
		return this._allRules;
		
	}
}
