/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.learn;

import lambdagen.data.LambdaPattern;
import lambdagen.data.LambdaProd;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaLearnCell{
	
	private static final long serialVersionUID = -8015531028034985690L;
	
	private LambdaProd _prod; //the prod associated with the cell, ie. one cell has one prod
	
    private int _N;
    
    private double[][] _beta;
    private double[][] _alpha;

    private double[][] _beta_w;
    private double[][][] _beta_X;
    private double[][][] _beta_Xw;
    private double[][][] _beta_wX;
    private double[][][] _beta_wXw;

    private double[][] _alpha_w;
    private double[][][] _alpha_X;
    private double[][][] _alpha_Xw;
    private double[][][] _alpha_wX;
    private double[][][] _alpha_wXw;
	
	public LambdaLearnCell(LambdaProd prod, int N){
		this._prod = prod;
		this._N = N;
		this.init();
	}
	
	private void init(){
		
        this._beta = new double[this._N][];
        this._alpha = new double[this._N][];
        for (int i = 0; i < this._N; i++) {
            this._beta[i] = new double[this._N - i];
            this._alpha[i] = new double[this._N - i];
        }
        
        int arity = this._prod.getArity();

        if (arity == 0) {
            this._beta_w = new double[this._N][];
            this._alpha_w = new double[this._N][];
            for (int i = 0; i < this._N; i++) {
                this._beta_w[i] = new double[this._N - i];
                this._alpha_w[i] = new double[this._N - i];
            }
        }

        if (arity == 1) {
            this._beta_X = new double[arity][][];
            this._alpha_X = new double[arity][][];
            
            for (int i = 0; i < arity; i++) {
                this._beta_X[i] = new double[this._N][];
                this._alpha_X[i] = new double[this._N][];

                for (int k = 0; k < this._N; k++) {
                    this._beta_X[i][k] = new double[this._N - k];
                    this._alpha_X[i][k] = new double[this._N - k];
                }
            }
            
                this._beta_Xw = new double[arity][][];
                this._alpha_Xw = new double[arity][][];
                
                for (int i = 0; i < arity; i++) {
                    this._beta_Xw[i] = new double[this._N][];
                    this._alpha_Xw[i] = new double[this._N][];

                    for (int k = 0; k < this._N; k++) {
                        this._beta_Xw[i][k] = new double[this._N - k];
                        this._alpha_Xw[i][k] = new double[this._N - k];
                    }
                }

                this._beta_wX = new double[arity][][];
            	this._alpha_wX = new double[arity][][];
                
                for (int i = 0; i < arity; i++) {
                    this._beta_wX[i] = new double[this._N][];
                    this._alpha_wX[i] = new double[this._N][];

                    for (int k = 0; k < this._N; k++) {
                        this._beta_wX[i][k] = new double[this._N - k];
                        this._alpha_wX[i][k] = new double[this._N - k];
                    }
                }
                
            	this._beta_wXw = new double[arity][][];
                this._alpha_wXw = new double[arity][][];
                
                for (int i = 0; i < arity; i++) {
                    this._beta_wXw[i] = new double[this._N][];
                    this._alpha_wXw[i] = new double[this._N][];

                    for (int k = 0; k < this._N; k++) {
                        this._beta_wXw[i][k] = new double[this._N - k];
                        this._alpha_wXw[i][k] = new double[this._N - k];
                    }
                }
        }
        
        if(arity == 2){
            this._beta_X = new double[arity][][];
            this._beta_Xw = new double[arity][][];
            this._beta_wX = new double[arity][][];
            this._beta_wXw = new double[arity][][];

            this._alpha_X = new double[arity][][];
            this._alpha_Xw = new double[arity][][];
            this._alpha_wX = new double[arity][][];
            this._alpha_wXw = new double[arity][][];

            for (int i = 0; i < arity; i++) {
                this._beta_X[i] = new double[this._N][];
                this._beta_Xw[i] = new double[this._N][];
                this._beta_wX[i] = new double[this._N][];
                this._beta_wXw[i] = new double[this._N][];

                this._alpha_X[i] = new double[this._N][];
                this._alpha_Xw[i] = new double[this._N][];
                this._alpha_wX[i] = new double[this._N][];
                this._alpha_wXw[i] = new double[this._N][];

                for (int k = 0; k < this._N; k++) {
                    this._beta_X[i][k] = new double[this._N - k];
                    this._beta_Xw[i][k] = new double[this._N - k];
                    this._beta_wX[i][k] = new double[this._N - k];
                    this._beta_wXw[i][k] = new double[this._N - k];

                    this._alpha_X[i][k] = new double[this._N - k];
                    this._alpha_Xw[i][k] = new double[this._N - k];
                    this._alpha_wX[i][k] = new double[this._N - k];
                    this._alpha_wXw[i][k] = new double[this._N - k];
                }
            }
        }
        
        if(arity >= 3 || arity < 0){
        	throw new RuntimeException("Invalid number for arity for gm "+"["+arity+"]"+this._prod.toString());
        }
        
	}
	
	public LambdaProd getProd(){
		return this._prod;
	}

    /* the main alpha/beta probabilities */
    public double getBetaProb(int bIndex, int eIndex) {
        return this.getProb(this._beta, bIndex, eIndex);
    }

    public double getAlphaProb(int bIndex, int eIndex) {
        return this.getProb(this._alpha, bIndex, eIndex);
    }

    public boolean addBetaProb(int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addAlphaProb(int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha, bIndex, eIndex, prob);
            return true;
        } else {
        	//throw new MRMTException("Fix: addAlpha");
        }
        return false;
    }

    public void resetAlphaProb(int bIndex, int eIndex) {
    	this._alpha[bIndex][eIndex - bIndex - 1] = 0;
    }

    /* the aux-beta probabilities */
    public double getBetaProb_aux(LambdaPattern p, int pos, int bIndex, int eIndex) {

        if (p == LambdaPattern._X) {
            return this.getBetaProb_X(pos, bIndex, eIndex);
        }
        if (p == LambdaPattern._wX) {
            return this.getBetaProb_wX(pos, bIndex, eIndex);
        }
        if (p == LambdaPattern._Xw) {
            return this.getBetaProb_Xw(pos, bIndex, eIndex);
        }
        if (p == LambdaPattern._wXw) {
            return this.getBetaProb_wXw(pos, bIndex, eIndex);
        }

        throw new RuntimeException("LambdaPattern " + p + " is not supported in getBetaProb_aux!");
    }

    public void addAlphaProb_aux(LambdaPattern p, int pos, int bIndex, int eIndex,
                                 double prob) {

        if (p == LambdaPattern._X) {
            this.addAlphaProb_X(pos, bIndex, eIndex, prob);
        } else if (p == LambdaPattern._wX) {
            this.addAlphaProb_wX(pos, bIndex, eIndex, prob);
        } else if (p == LambdaPattern._Xw) {
            this.addAlphaProb_Xw(pos, bIndex, eIndex, prob);
        } else if (p == LambdaPattern._wXw) {
            this.addAlphaProb_wXw(pos, bIndex, eIndex, prob);
        } else {
            throw new RuntimeException("LambdaPattern " + p + " is not supported in addAlphaProb_aux!");
        }
    }

    public double getBetaProb_w(int bIndex, int eIndex) {
        return this.getProb(this._beta_w, bIndex, eIndex);
    }

    public double getBetaProb_X(int pos, int bIndex, int eIndex) {
        return this.getProb(this._beta_X, pos, bIndex, eIndex);
    }

    public double getBetaProb_wX(int pos, int bIndex, int eIndex) {
        return this.getProb(this._beta_wX, pos, bIndex, eIndex);
    }

    public double getBetaProb_Xw(int pos, int bIndex, int eIndex) {
        return this.getProb(this._beta_Xw, pos, bIndex, eIndex);
    }

    public double getBetaProb_wXw(int pos, int bIndex, int eIndex) {
        return this.getProb(this._beta_wXw, pos, bIndex, eIndex);
    }

    // adding the inside probabilities
    public boolean addBetaProb_w(int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta_w, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addBetaProb_X(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta_X, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addBetaProb_wX(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta_wX, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addBetaProb_Xw(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta_Xw, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addBetaProb_wXw(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._beta_wXw, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    /* the aux-alpha probabilities */
    public double getAlphaProb_w(int bIndex, int eIndex) {
        return this.getProb(this._alpha_w, bIndex, eIndex);
    }

    public double getAlphaProb_X(int pos, int bIndex, int eIndex) {
        return this.getProb(this._alpha_X, pos, bIndex, eIndex);
    }

    public double getAlphaProb_wX(int pos, int bIndex, int eIndex) {
        return this.getProb(this._alpha_wX, pos, bIndex, eIndex);
    }

    public double getAlphaProb_Xw(int pos, int bIndex, int eIndex) {
        return this.getProb(this._alpha_Xw, pos, bIndex, eIndex);
    }

    public double getAlphaProb_wXw(int pos, int bIndex, int eIndex) {
        return this.getProb(this._alpha_wXw, pos, bIndex, eIndex);
    }

    // adding the outside probabilities
    public boolean addAlphaProb_w(int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha_w, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addAlphaProb_X(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha_X, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addAlphaProb_wX(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha_wX, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addAlphaProb_Xw(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha_Xw, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }

    public boolean addAlphaProb_wXw(int pos, int bIndex, int eIndex, double prob) {
        if (prob > 0) {
            this.addProb(this._alpha_wXw, pos, bIndex, eIndex, prob);
            return true;
        }
        return false;
    }
    
    /* helper methods */
    private double getProb(double[][] array, int bIndex, int eIndex) {
    	if(array == null){
    		return 0;
    	}
        if (eIndex - bIndex <= 0) {
            return 0;
        }
        return array[bIndex][eIndex - bIndex - 1];
    }

    private double getProb(double[][][] array, int pos, int bIndex, int eIndex) {
    	if(array == null){
    		return 0;
    	}
        if (eIndex - bIndex <= 0) {
            return 0;
        }
        return array[pos][bIndex][eIndex - bIndex - 1];
    }

    private boolean addProb(double[][] array, int bIndex, int eIndex, double prob) {
        if (eIndex - bIndex <= 0) {
            return false;
        }
        if (prob > 0) {
            array[bIndex][eIndex - bIndex - 1] += prob;
            return true;
        }
        return false;
    }

    private boolean addProb(double[][][] array, int pos, int bIndex, int eIndex,
                            double prob) {
        if (eIndex - bIndex <= 0) {
            return false;
        }
        if (prob > 0) {
            array[pos][bIndex][eIndex - bIndex - 1] += prob;
            return true;
        }
        return false;
    }

}
