/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.main;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import lambdacalc.logic.TypeEvaluationException;
import lambdagen.data.Instance;
import lambdagen.gen.LambdaLM;
import lambdagen.util.Config;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaTrainLM {

	private String _modeldir;
	private String _srilm_path;
	private short _n;//lm order
	private String _lm_file;
	
	public static void main(String args[]) throws FileNotFoundException, IOException, ClassNotFoundException, TypeEvaluationException{

		if(args.length == 0){
			System.err.println("Usage: java LambdaTrainLM <cfg_file>");
			System.exit(1);
		}
		
		Config cfg = new Config(args[0]);
		LambdaTrainLM lm = new LambdaTrainLM(cfg);
		System.err.println(cfg);
		lm.train_lm();
		
	}

	public LambdaTrainLM(Config cfg){
		this._modeldir = cfg.readArgStr("modeldir", "");
		this._n = (short)cfg.readArgInt("lmorder", 3);
		this._lm_file = cfg.readArgStr("lmfile", "");
		this._srilm_path = cfg.readArgStr("srilm", "");
	}
	
	public void train_lm() throws FileNotFoundException, IOException, ClassNotFoundException, TypeEvaluationException{
		
		LambdaLM llm = new LambdaLM(_lm_file, _n);
		
		String instance_train_file = this._modeldir+"/train.instances";
		ArrayList<Instance> instances_train = LambdaReader.readInstances(instance_train_file);
		
		String instance_init_file = this._modeldir+"/init.instances";
		ArrayList<Instance> instances_init = LambdaReader.readInstances(instance_init_file);
		
		ArrayList<Instance> instances = new ArrayList<Instance>();
		for(int i = 0 ; i<instances_train.size(); i++){
			instances.add(instances_train.get(i));
		}
		for(int i = 0 ; i<instances_init.size(); i++){
			instances.add(instances_init.get(i));
		}
		
		llm.trainSrilm(_srilm_path, instances, _modeldir);
	}
	
}
