/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.phrase;

import java.util.Arrays;

import lambdagen.data.LambdaPattern;
import lambdagen.grammar.LambdaGRule;
import lambdagen.util.Logger;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaPhraseCell {
	
    public LambdaGRule _rule;
    private int _N;
    private int _K;
    
    private LambdaPhraseSpan[][][] _max;
    
    private LambdaPhraseSpan[][][] _max_w;
    
    private LambdaPhraseSpan[][][] _max_X;
    private LambdaPhraseSpan[][][] _max_Xw;
    private LambdaPhraseSpan[][][] _max_wX;
    private LambdaPhraseSpan[][][] _max_wXw;
    
    private LambdaPhraseSpan[][][] _max_Y;
    private LambdaPhraseSpan[][][] _max_Yw;
    private LambdaPhraseSpan[][][] _max_wY;
    private LambdaPhraseSpan[][][] _max_wYw;
    
    public LambdaPhraseCell(LambdaGRule rule, int N, int K) {
        this._rule = rule;
        this._N = N;
        this._K = K;
        this.init();
    }
    
    private void init() {
        this._max = new LambdaPhraseSpan[this._N][][];
        for(int len = 1; len <= this._N; len++){
        	this._max[len-1] = new LambdaPhraseSpan[this._N - len + 1][this._K];
            for (int bIndex = 0; bIndex < this._max[len-1].length; bIndex++) {
            	Arrays.fill(this._max[len-1][bIndex], LambdaPhraseSpan._empty);
            }
        }
    }
    
    private void initArray(LambdaPattern p, int len){
    	if(p == LambdaPattern._w){
    		if(this._max_w == null)
    			this._max_w = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_w[k] = new LambdaPhraseSpan[this._N - k][1];
            for (int j = 0; j < this._N - len + 1; j++) {
            	this._max_w[k][j][0]=LambdaPhraseSpan._empty;
            }
    	} else if(p == LambdaPattern._X){
    		if(this._max_X == null)
    			this._max_X = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_X[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_X[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._wX){
    		if(this._max_wX == null)
    			this._max_wX = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_wX[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_wX[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._Xw){
    		if(this._max_Xw == null)
    			this._max_Xw = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_Xw[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_Xw[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._wXw){
    		if(this._max_wXw == null)
    			this._max_wXw = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_wXw[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_wXw[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._Y){
    		if(this._max_Y == null)
    			this._max_Y = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_Y[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_Y[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._wY){
    		if(this._max_wY == null)
    			this._max_wY = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_wY[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_wY[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._Yw){
    		if(this._max_Yw == null)
    			this._max_Yw = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_Yw[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_Yw[k][j], LambdaPhraseSpan._empty);
            }
    	} else if(p == LambdaPattern._wYw){
    		if(this._max_wYw == null)
    			this._max_wYw = new LambdaPhraseSpan[this._N][][];
    		int k = len - 1;
            this._max_wYw[k] = new LambdaPhraseSpan[this._N - k][this._K];
            for (int j = 0; j < this._N - k; j++) {
            	Arrays.fill(this._max_wYw[k][j], LambdaPhraseSpan._empty);
            }
    	} else {
    		Logger.error("Invalid LambdaPattern for initialization!"+p);
    	}
    }
    
    /*major methods*/
    public boolean setMaxProb(int bIndex, int eIndex, double prob, LambdaPhraseSpan[] c_spans, LambdaPattern pattern) {
    	
    	if(prob<=0){
    		return false;
    	}
    	
    	if(this._rule.getArity() == 0 && c_spans == null){
    	}else if (this._rule.getArity() != c_spans.length) {
            throw new RuntimeException("Mismatch between the arity and the number of children:"+this._rule.getArity()+";"+c_spans.length);
        }
        
        LambdaPhraseSpan newSpan = new LambdaPhraseSpan(this._rule.getId(), bIndex, eIndex, prob);
        newSpan.setChildren(c_spans);
        
        return this.setSpan(this._max[eIndex-bIndex-1], bIndex, newSpan);
        
    }
    
    public LambdaPhraseSpan getMaxSpan(int bIndex, int eIndex) {
        LambdaPhraseSpan[] spans = this.getSpan(this._max[eIndex-bIndex], bIndex);
        return spans[spans.length - 1];
    }
    
    public LambdaPhraseSpan[] getMaxSpans(int bIndex, int eIndex) {
        return this.getSpan(this._max[eIndex-bIndex-1], bIndex);
    }

    /*aux methods*/
    public boolean setMaxProb_p(LambdaPattern p, int bIndex, int eIndex, double prob, LambdaPhraseSpan c_span) {
    	
    	if(prob <= 0){
    		return false;
    	}

        LambdaPhraseSpan newEntry = new LambdaPhraseSpan(this._rule.getId(), bIndex, eIndex, prob);
        newEntry.setChildren(new LambdaPhraseSpan[] {c_span});
        if(this.getArray(p, eIndex-bIndex)==null){
        	this.initArray(p, eIndex-bIndex);
        }
        return this.setSpan(this.getArray(p, eIndex-bIndex), bIndex, newEntry);
    }

    public LambdaPhraseSpan[] getMaxProb_p(LambdaPattern p, int bIndex, int eIndex) {
    	if(this.getArray(p, eIndex - bIndex)== null){
    		return new LambdaPhraseSpan[]{LambdaPhraseSpan._empty};
        	//this.initArray(p);
    	}
		return this.getSpan(this.getArray(p, eIndex- bIndex), bIndex);
    }

    private LambdaPhraseSpan[] getSpan(LambdaPhraseSpan[][] array, int bIndex) {
        return array[bIndex];
    }

    private boolean setSpan(LambdaPhraseSpan[][] array, int bIndex, LambdaPhraseSpan span) {
        if (span.getProb() == 0) {
            return false;
        }

        LambdaPhraseSpan[] arr = array[bIndex];
        int index = Arrays.binarySearch(arr, span);
        if (index < 0) {
            index = -1 - index;
        }

        if (index == 0) {
            return false;
        }

        index = index - 1;

        //insertion
        for (int k = 0; k <= index - 1; k++) {
            arr[k] = arr[k + 1];
        }
        arr[index] = span;
        return true;
    }
    
    private LambdaPhraseSpan[][] getArray(LambdaPattern p, int len){
    
    	if(p == LambdaPattern._w){
    		if(this._max_w==null){
    			return null;
    		}
    		return this._max_w[len-1];
    	} else if(p == LambdaPattern._X){
    		if(this._max_X==null){
    			return null;
    		}
    		return this._max_X[len-1];
    	} else if(p == LambdaPattern._wX){
    		if(this._max_wX==null){
    			return null;
    		}
    		return this._max_wX[len-1];
    	} else if(p == LambdaPattern._Xw){
    		if(this._max_Xw==null){
    			return null;
    		}
    		return this._max_Xw[len-1];
    	} else if(p == LambdaPattern._wXw){
    		if(this._max_wXw==null){
    			return null;
    		}
    		return this._max_wXw[len-1];
    	} else if(p == LambdaPattern._Y){
    		if(this._max_Y==null){
    			return null;
    		}
    		return this._max_Y[len-1];
    	} else if(p == LambdaPattern._wY){
    		if(this._max_wY==null){
    			return null;
    		}
    		return this._max_wY[len-1];
    	} else if(p == LambdaPattern._Yw){
    		if(this._max_Yw==null){
    			return null;
    		}
    		return this._max_Yw[len-1];
    	} else if(p == LambdaPattern._wYw){
    		if(this._max_wYw==null){
    			return null;
    		}
    		return this._max_wYw[len-1];
    	} else {
    		throw new RuntimeException("Invalid LambdaPattern in ParseCell!");
    	}
    }
}
