/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.phrase;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaPhraseSpanPair implements Comparable<LambdaPhraseSpanPair> {
	
    private LambdaPhraseSpan _entry1;
    private LambdaPhraseSpan _entry2;
    private double _score;
    private int[] _position;

    public LambdaPhraseSpanPair(LambdaPhraseSpan[] entry1, LambdaPhraseSpan[] entry2, int i, int j) {
        this._entry1 = entry1[i];
        this._entry2 = entry2[j];
    	this._score = this._entry1.getProb() * this._entry2.getProb();
        this._position = new int[] {i, j};
    }
    
    public boolean isPositive(){
    	return this._score > 0;
    }

    public int[] getPosition() {
        return this._position;
    }

    @Override
	public int compareTo(LambdaPhraseSpanPair pair) {
        return this._score > pair._score ? -1 : this._score == pair._score ? 0 : +1;
    }

    public LambdaPhraseSpan getFirstEntry() {
        return this._entry1;
    }

    public LambdaPhraseSpan getSecondEntry() {
        return this._entry2;
    }

    public LambdaPhraseSpan[] toArray() {
        return new LambdaPhraseSpan[] {this._entry1, this._entry2};
    }

}