/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.StringTokenizer;

/**
 * @author Wei LU
 * @version 1.0
 */

public class Config{
	
	private String _filename;
	private ArrayList<String> _lines;
	private ArrayList<String> _trackedLines;
	
	public Config(String filename){
		this._filename = filename;
		readFile();
	}
	
	private void readFile(){
		try{
			Scanner scan = new Scanner(new File(this._filename));
			this._lines = new ArrayList<String>();
			while(scan.hasNextLine()){
				String line = scan.nextLine();
				this._lines.add(line);
			}
			this._trackedLines = new ArrayList<String>();
		}catch(FileNotFoundException e){
			System.err.println("Error: the configuration file "+this._filename+" is not found!");
			System.exit(1);
		}
	}
	
	public String readArgStr(String arg, String def){
		for(int i =0; i<this._lines.size(); i++){
			String line = this._lines.get(i);
			if(line.isEmpty()){
				continue;
			}
			StringTokenizer st = new StringTokenizer(line);
			String arg0 = st.nextToken();
			if(arg0.equalsIgnoreCase(arg)){
				String val = st.nextToken();
				this._trackedLines.add(arg0+": "+val);
				return val;
			}
		}
		return def;
	}
	
	public int readArgInt(String arg, int def){
		for(int i =0; i<this._lines.size(); i++){
			String line = this._lines.get(i).trim();
			if(line.isEmpty()){
				continue;
			}
			StringTokenizer st = new StringTokenizer(line);
			String arg0 = st.nextToken();
			if(arg0.equalsIgnoreCase(arg)){
				String val = st.nextToken();
				this._trackedLines.add(arg0+": "+val);
				return Integer.parseInt(val);
			}
		}
		this._trackedLines.add(arg+" "+def+"\t[warning: arg not found in config file - use default]");
		return def;
	}
	
	public boolean readArgBool(String arg, boolean def){
		for(int i =0; i<this._lines.size(); i++){
			String line = this._lines.get(i).trim();
			if(line.isEmpty()){
				continue;
			}
			StringTokenizer st = new StringTokenizer(line);
			String arg0 = st.nextToken();
			if(arg0.equalsIgnoreCase(arg)){
				String val = st.nextToken();
				this._trackedLines.add(arg0+": "+val);
				return Boolean.parseBoolean(val);
			}
		}
		this._trackedLines.add(arg+" "+def+"\t[warning: arg not found in config file - use default]");
		return def;
	}
	
	
	
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		
		sb.append("-Configurations-");
		sb.append('\n');
		for(int i = 0; i<this._trackedLines.size(); i++){
			sb.append(this._trackedLines.get(i));
			sb.append('\n');
		}
		sb.append("---------------");
		sb.append('\n');
		
		return sb.toString();
	}
}
