/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.util;

import java.util.ArrayList;

/**
 * @author Wei LU
 * @version 1.0
 */

public class Sequence<X> implements Comparable<Sequence<X>>{
	
	private ArrayList<X> _x1;
	private double _val;
	
	public Sequence(ArrayList<X> x1){
		this._x1 = x1;
		this._val = 0;
	}
	
	public Sequence(ArrayList<X> x1, double val){
		this._x1 = x1;
		this._val = val;
	}
	
	public double getVal(){
		return this._val;
	}
	
	public ArrayList<X> toArrayList(){
		return this._x1;
	}
	
	@Override
	public int hashCode(){
		if(this._x1.size()==0){
			return 0;
		}
		int result = this._x1.get(0).hashCode();
		for(int i =1; i<this._x1.size(); i++){
			result = result ^ this._x1.get(i).hashCode();
		}
		return result;
	}
	
	@Override
	public boolean equals(Object o){
		if(o instanceof Sequence){
			@SuppressWarnings("unchecked")
			Sequence<X> seq = (Sequence<X>)o;
			return this.compareTo(seq) == 0? true : false;
		}
		return false;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public int compareTo(Sequence<X> seq) {
		if(this._x1.size() > seq._x1.size()){
			return +1;
		} else if(this._x1.size() < seq._x1.size()){
			return -1;
		}
		for(int i =0; i<this._x1.size(); i++){
			int val = ((Comparable)this._x1.get(i)).compareTo(seq._x1.get(i));
			if(val != 0){
				return val;
			}
		}
		return 0;
	}

}
