/**
 *  A Probabilistic Forest-to-String Model for Language Generation from Typed Lambda Calculus Expressions
 *
 *  Copyright (C) 2011  LU, Wei (luwei@statnlp.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package lambdagen.viterbi;

import java.util.PriorityQueue;

/**
 * @author Wei LU
 * @version 1.0
 */

public class LambdaAlignSquare {

	    private LambdaAlignSpan[] _spans1;
	    private LambdaAlignSpan[] _spans2;
	    
	    private int[][] _matrix; //the matrix showing which entries are marked, and which are not.
	    
	    private PriorityQueue<LambdaAlignSpanPair> _waiting_list; //the list of next possible ones
	    
	    public LambdaAlignSquare(LambdaAlignSpan[] spans1, LambdaAlignSpan[] spans2) {
	        this._spans1 = spans1;
	        this._spans2 = spans2;
	        this._waiting_list = new PriorityQueue<LambdaAlignSpanPair>();

	        LambdaAlignSpanPair root_pair = new LambdaAlignSpanPair(this._spans1, this._spans2, this._spans1.length - 1, this._spans2.length - 1);
	        this._waiting_list.offer(root_pair);

	        this._matrix = new int[this._spans1.length][this._spans2.length];
	    }
	    
	    public LambdaAlignSpan[] getNext() {
	        LambdaAlignSpanPair result = this._waiting_list.poll();
	        if (result == null || !result.isPositive()) {
	            return null;
	        }
	        int i = result.getPosition()[0];
	        int j = result.getPosition()[1];

	        this._matrix[i][j] = -1; //mark it as retrieved

	        if (j - 1 >= 0 && this._matrix[i][j - 1] == 0) {
	            LambdaAlignSpanPair pair = new LambdaAlignSpanPair(this._spans1, this._spans2, i, j - 1);
	            this._waiting_list.offer(pair);
	            this._matrix[i][j - 1] = 1; //shaded
	        }
	        if (i - 1 >= 0 && this._matrix[i - 1][j] == 0) {
	            LambdaAlignSpanPair pair = new LambdaAlignSpanPair(this._spans1, this._spans2, i - 1, j);
	            this._waiting_list.offer(pair);
	            this._matrix[i - 1][j] = 1; //shaded
	        }

	        LambdaAlignSpan[] res = result.toArray();
	        return res;
	    }
	    
}
